@@ -22,72 +22,68 @@
 #'   confocal microscope operating in the analog mode. Microsc. Res. Tech., 71,
 #'   1:69-81. \doi{10.1002/jemt.20526}.
 #'
-#'   Hur K-H, Macdonald PJ, Berk S, Angert CI, Chen Y, Mueller JD (2014)
-#'   Quantitative Measurement of Brightness from Living Cells in the Presence of
-#'   Photodepletion. PLoS ONE 9(5): e97440. \doi{10.1371/journal.pone.0097440}.
-#'
 #' @examples
 #' img <- ijtiff::read_tif(system.file('extdata', '50.tif', package = 'nandb'))
 #' ijtiff::display(img[, , 1, 1])
-#' b <- brightness(img, "e", tau = NA, thresh = "Huang")
-#' b <- brightness(img, "B", tau = 10, thresh = "tri")
+#' b <- brightness(img, "e", thresh = "Huang")
+#' b <- brightness(img, "B", thresh = "tri")
 #' @export
-brightness <- function(img, def, tau = NULL,
-                       thresh = NULL, filt = NULL, correct = FALSE,
-                       s = 1, offset = 0, readout_noise = 0, parallel = FALSE) {
+brightness <- function(img, def, thresh = NULL, detrend = FALSE, quick = FALSE,
+                       filt = NULL,
+                       s = 1, offset = 0, readout_noise = 0,
+                       parallel = FALSE) {
   checkmate::assert_string(def)
   if (startsWith("epsilon", tolower(def))) def <- "epsilon"
   if (def == "b") def <- "B"
-  if (! def %in% c("epsilon", "B"))
-    stop("'def' must be one of 'B' or 'epsilon'.")
+  if (!def %in% c("epsilon", "B")) {
+    custom_stop("`def` must be one of 'B' or 'epsilon'.",
+                "You have used `def = '{def}'`.")
+  }
   img %<>% nb_get_img()
   d <- dim(img)
   n_ch <- dplyr::if_else(length(d) == 3, 1L, d[3])
-  if (n_ch == 1 && length(d) == 4) img %<>% {.[, , 1, ]}
+  if (n_ch == 1 && length(d) == 4) {
+    dim(img) %<>% {
+      .[-3]
+    }
+  }
   thresh %<>% extend_for_all_chs(n_ch)
-  tau %<>% extend_for_all_chs(n_ch)
+  detrend %<>% extend_for_all_chs(n_ch)
   if (!is.null(filt)) filt %<>% fix_filt()
   filt %<>% extend_for_all_chs(n_ch) %>% unlist() %>% as.character()
-  tau_atts <- extend_for_all_chs(rlang::set_attrs(NA, auto = FALSE),
-                                 n_ch)
+  swaps_atts <- extend_for_all_chs(
+    rlang::set_attrs(NA, auto = FALSE),
+    n_ch
+  )
   thresh_atts <- extend_for_all_chs(NA, n_ch)
   if (n_ch == 1) {
-    if (!is.na(thresh)) {
-      img %<>% autothresholdr::mean_stack_thresh(method = thresh,
-                                                 ignore_na = TRUE)
+    if (!is.na(thresh[[1]])) {
+      img %<>% autothresholdr::mean_stack_thresh(
+        method = thresh[[1]],
+        ignore_na = TRUE
+      )
       thresh_atts <- attr(img, "thresh")
     }
-    if (!is.na(tau)) {
-      img %<>% detrendr::img_detrend_exp(tau, parallel = parallel,
-                                         purpose = "FFS")
-      tau_atts <- attr(img, "parameter")
-      attr(tau_atts, "auto") <- attr(img, "auto")
+    if (detrend[[1]]) {
+      img %<>% detrendr::img_detrend_rh(quick = quick)
+      swaps_atts <- attr(img, "parameter")
+      attr(swaps_atts, "auto") <- attr(img, "auto")
     }
     out <- (detrendr::var_pillars(img, parallel = parallel) - readout_noise) /
       (detrendr::mean_pillars(img, parallel = parallel) - offset)
-    if (correct) {
-      out %<>% {. / s - 1}
-      if (length(dim(img)) == 4) img %<>% {.[, , 1, ]}
-      frame_means <- detrendr::mean_frames(img, na_rm = TRUE)
-      if (length(unique(frame_means)) >= 4) {
-        smooth_spline <- stats::smooth.spline(frame_means) %>%
-          getElement("y")
-      } else {
-        data <- dplyr::tibble(x = seq_along(frame_means),
-                              y = frame_means)
-        robust_fit <- MASS::rlm(y ~ x, data)
-        smooth_spline <- stats::predict(robust_fit, data[c(1, nrow(data)), ])
+    if (def == "epsilon") {
+      out %<>% {
+        . / s - 1
       }
-      f_d <- smooth_spline %>% {(.[[1]] - dplyr::last(.)) / .[[1]]}
-      out %<>% {. / (1 - f_d / 2) - ((f_d / 2) / (1 - f_d / 2))}
-      if (def == "B") out %<>% {s * (1 + .)}
-    } else if (def == "epsilon") {
-      out %<>% {. / s - 1}
     }
-    if (length(dim(out)) > 2) dim(out) %<>% {.[1:2]}
-    if (!is.na(filt)) {
+    if (length(dim(out)) > 2) {
+      dim(out) %<>% {
+        .[1:2]
+      }
+    }
+    if (!is.na(filt[[1]])) {
       checkmate::assert_string(filt)
-      if (filt == "median") {
+      if (filt[[1]] == "median") {
         out %<>% median_filter(na_count = TRUE)
       } else {
         out %<>% smooth_filter(na_count = TRUE)
@@ -96,19 +92,23 @@ brightness <- function(img, def, tau = NULL,
   } else {
     out <- img[, , , 1]
     thresh_atts <- list()
-    tau_atts <- list()
+    swaps_atts <- list()
     for (i in seq_len(n_ch)) {
-      out_i <- brightness(img[, , i, ], def = def, tau = tau[[i]],
-                          thresh = thresh[[i]], filt = filt[[i]],
-                          correct = correct,
-                          s = s, offset = offset, readout_noise = readout_noise,
-                          parallel = parallel)
+      out_i <- brightness(img[, , i, , drop = FALSE],
+        def = def, thresh = thresh[[i]],
+        detrend = detrend[[i]], quick = quick,
+        filt = filt[[i]],
+        s = s, offset = offset, readout_noise = readout_noise,
+        parallel = parallel
+      )
       out[, , i] <- out_i
       thresh_atts[[i]] <- attr(out_i, "thresh")
-      tau_atts[[i]] <- attr(out_i, "tau")
+      swaps_atts[[i]] <- attr(out_i, "swaps")
     }
   }
-  brightness_img(out, def, thresh_atts, tau_atts, filt)
+  brightness_img(out,
+                 def = def,
+                 thresh = thresh_atts, swaps = swaps_atts, filt = filt)
 }
 
 #' Create a brightness time-series.
@@ -117,13 +117,18 @@ brightness <- function(img, def, tau = NULL,
 #' create one brightness image, the next `frames_per_set` of them to create the
 #' next brightness image and so on to get a time-series of brightness images.
 #'
+#' This may discard some images, for example if 175 frames are in the input and
+#' `frames_per_set = 50`, then the last 25 are discarded. If detrending is
+#' selected, it is performed on the whole image stack before the sectioning is
+#' done for calculation of numbers.
+#'
 #' @param frames_per_set The number of frames with which to calculate the
 #'   successive brightnesses.
-#'
-#' This may discard some images, for example if 175 frames are in the input and
-#' `frames_per_set = 50`, then the last 25 are discarded. If bleaching
-#' correction is selected, it is performed on the whole image stack before the
-#' sectioning is done for calculation of numbers.
+#' @param overlap A boolean. If `TRUE`, the windows used to calculate number are
+#'   overlapped, if `FALSE`, they are not. For example, for a 20-frame image
+#'   series with 5 frames per set, if the windows are not overlapped, then the
+#'   frame sets used are 1-5, 6-10, 11-15 and 16-20; whereas if they are
+#'   overlapped, the frame sets are 1-5, 2-6, 3-7, 4-8 and so on up to 16-20.
 #'
 #' @inheritParams brightness
 #' @inheritParams number
@@ -141,12 +146,11 @@ brightness <- function(img, def, tau = NULL,
 #'
 #' @examples
 #' img <- ijtiff::read_tif(system.file('extdata', '50.tif', package = 'nandb'))
-#' bts <- brightness_timeseries(img, "e", frames_per_set = 20,
-#'                               tau = NA, thresh = "Huang", parallel = 2)
+#' bts <- brightness_timeseries(img, "e", frames_per_set = 20, thresh = "Huang")
 #' @export
-brightness_timeseries <- function(img, def, frames_per_set,
-                                  tau = NULL, thresh = NULL,
-                                  filt = NULL, correct = FALSE,
+brightness_timeseries <- function(img, def, frames_per_set, overlap = FALSE,
+                                  thresh = NULL, detrend = FALSE, quick = FALSE,
+                                  filt = NULL,
                                   s = 1, offset = 0,
                                   readout_noise = 0, parallel = FALSE) {
   if (startsWith("epsilon", tolower(def))) def <- "epsilon"
@@ -154,42 +158,71 @@ brightness_timeseries <- function(img, def, frames_per_set,
   img %<>% nb_get_img()
   d <- dim(img)
   n_ch <- dplyr::if_else(length(d) == 3, 1L, d[3])
-  if (n_ch == 1 && length(d) == 4) img %<>% {.[, , 1, ]}
+  if (n_ch == 1 && length(d) == 4) {
+    dim(img) %<>% {
+      .[-3]
+    }
+  }
   thresh %<>% extend_for_all_chs(n_ch)
-  tau %<>% extend_for_all_chs(n_ch)
+  detrend %<>% extend_for_all_chs(n_ch)
   if (!is.null(filt)) filt %<>% fix_filt()
   filt %<>% extend_for_all_chs(n_ch) %>% unlist() %>% as.character()
-  tau_atts <- extend_for_all_chs(rlang::set_attrs(NA, auto = FALSE),
-                                 n_ch)
+  swaps_atts <- extend_for_all_chs(
+    rlang::set_attrs(NA, auto = FALSE),
+    n_ch
+  )
   thresh_atts <- extend_for_all_chs(NA, n_ch)
   if (n_ch == 1) {
     frames <- dim(img)[3]
     if (frames < frames_per_set) {
-      stop("You have selected ", frames_per_set, " frames per set, ",
-           "but there are only ", frames, " frames in total.")
+      custom_stop("
+        You have selected {frames_per_set} frames per set,
+        but there are only {frames}, frames in total.
+        ", "
+        Please select less than {frames} frames per set.
+        "
+      )
     }
-    sets <- frames %/% frames_per_set
-    if (!is.na(thresh)) {
-      img %<>% autothresholdr::mean_stack_thresh(method = thresh,
-                                                 ignore_na = TRUE)
+    if (!is.na(thresh[[1]])) {
+      img %<>% autothresholdr::mean_stack_thresh(
+        method = thresh[[1]],
+        ignore_na = TRUE
+      )
       thresh_atts <- attr(img, "thresh")
       img <- img[, , 1, ]
     }
-    if (!is.na(tau)) {
-      img %<>% detrendr::img_detrend_exp(tau = tau, purpose = "FFS",
-                                         parallel = parallel)
-      tau_atts <- attr(img, "parameter")
-      attr(tau_atts, "auto") <- attr(img, "auto")
+    if (detrend[[1]]) {
+      img %<>% detrendr::img_detrend_rh(quick = quick)
+      swaps_atts <- attr(img, "parameter")
+      attr(swaps_atts, "auto") <- attr(img, "auto")
       img <- img[, , 1, ]
     }
-    out <- img[, , seq_len(sets)]
-    if (sets == 1) dim(out) %<>% c(1)
-    for (i in seq_len(sets)) {
-      indices_i <- seq((i - 1) * frames_per_set + 1, i * frames_per_set)
-      out[, , i] <- brightness(img[, , indices_i], def = def, filt = filt,
-                               correct = correct, s = s, offset = offset,
-                               readout_noise = readout_noise,
-                               parallel = parallel)
+    if (overlap) {
+      sets <- frames - frames_per_set + 1
+      out <- img[, , seq_len(sets)]
+      if (sets == 1) dim(out) %<>% c(1)
+      for (i in seq_len(sets)) {
+        indices_i <- seq(i, i + frames_per_set - 1)
+        out[, , i] <- brightness(img[, , indices_i],
+                                 def = def, detrend = FALSE, filt = filt,
+                                 s = s, offset = offset,
+                                 readout_noise = readout_noise,
+                                 parallel = parallel
+        )
+      }
+    } else {
+      sets <- frames %/% frames_per_set
+      out <- img[, , seq_len(sets)]
+      if (sets == 1) dim(out) %<>% c(1)
+      for (i in seq_len(sets)) {
+        indices_i <- seq((i - 1) * frames_per_set + 1, i * frames_per_set)
+        out[, , i] <- brightness(img[, , indices_i],
+                                 def = def, detrend = FALSE, filt = filt,
+                                 s = s, offset = offset,
+                                 readout_noise = readout_noise,
+                                 parallel = parallel
+        )
+      }
     }
   } else {
     frames <- d[4]
@@ -197,27 +230,32 @@ brightness_timeseries <- function(img, def, frames_per_set,
     out <- img[, , , seq_len(sets)]
     if (sets == 1) dim(out) %<>% c(1)
     thresh_atts <- list()
-    tau_atts <- list()
+    swaps_atts <- list()
     for (i in seq_len(n_ch)) {
-      out_i <- brightness_timeseries(img[, , i, ], def = def,
-                                     frames_per_set = frames_per_set,
-                                     tau = tau[[i]], thresh = thresh[[i]],
-                                     filt = filt[[i]], offset = offset,
-                                     correct = correct,
-                                     readout_noise = readout_noise,
-                                     parallel = parallel)
+      out_i <- brightness_timeseries(img[, , i, , drop = FALSE],
+        def = def,
+        frames_per_set = frames_per_set,
+        thresh = thresh[[i]],
+        detrend = detrend[[i]],
+        filt = filt[[i]],
+        s = s, offset = offset, readout_noise = readout_noise,
+        parallel = parallel
+      )
       out[, , i, ] <- out_i
       thresh_atts[[i]] <- attr(out_i, "thresh")
-      tau_atts[[i]] <- attr(out_i, "tau")
+      swaps_atts[[i]] <- attr(out_i, "swaps")
     }
   }
-  brightness_ts_img(out, def = def, frames_per_set = frames_per_set,
-                    thresh = thresh_atts, tau = tau_atts, filt = filt)
+  brightness_ts_img(out,
+    def = def, frames_per_set = frames_per_set, overlapped = overlap,
+    thresh = thresh_atts, swaps = swaps_atts, filt = filt
+  )
 }
 
-brightness_file <- function(path, def, tau = NULL,
-                            thresh = NULL, filt = NULL,
-                            correct = FALSE,
+brightness_file <- function(path, def,
+                            thresh = NULL,
+                            detrend = FALSE, quick = FALSE,
+                            filt = NULL,
                             s = 1, offset = 0, readout_noise = 0,
                             parallel = FALSE) {
   checkmate::assert_file_exists(path)
@@ -229,10 +267,13 @@ brightness_file <- function(path, def, tau = NULL,
     setwd(dir)
     path %<>% filesstrings::str_after_last("/")
   }
-  b <- brightness(path, def, tau = tau,
-                  thresh = thresh, filt = filt, correct = correct,
-                  s = s, offset = offset, readout_noise = readout_noise,
-                  parallel = parallel)
+  b <- brightness(path, def,
+    thresh = thresh,
+    detrend = detrend, quick = quick,
+    filt = filt,
+    s = s, offset = offset, readout_noise = readout_noise,
+    parallel = parallel
+  )
   suppressMessages(filesstrings::create_dir("brightness"))
   path %<>% filesstrings::before_last_dot() %>%
     paste0("brightness", "/", ., make_nb_filename_ending(b)) %>%
@@ -241,8 +282,10 @@ brightness_file <- function(path, def, tau = NULL,
 }
 
 brightness_timeseries_file <- function(path, def, frames_per_set,
-                                       tau = NULL, thresh = NULL,
-                                       filt = NULL, correct = FALSE,
+                                       overlap = FALSE,
+                                       thresh = NULL,
+                                       detrend = FALSE, quick = FALSE,
+                                       filt = NULL,
                                        s = 1, offset = 0,
                                        readout_noise = 0,
                                        parallel = FALSE) {
@@ -257,12 +300,15 @@ brightness_timeseries_file <- function(path, def, frames_per_set,
     setwd(dir)
     path %<>% filesstrings::str_after_last("/")
   }
-  bts <- brightness_timeseries(path, def, frames_per_set = frames_per_set,
-                               tau = tau,
-                               thresh = thresh, filt = filt,
-                               correct = correct, s = s, offset = offset,
-                               readout_noise = readout_noise,
-                               parallel = parallel)
+  bts <- brightness_timeseries(path, def,
+    frames_per_set = frames_per_set, overlap = overlap,
+    thresh = thresh,
+    detrend = detrend, quick = quick,
+    filt = filt,
+    s = s, offset = offset,
+    readout_noise = readout_noise,
+    parallel = parallel
+  )
   suppressMessages(filesstrings::create_dir("brightness_timeseries"))
   path %<>% filesstrings::before_last_dot() %>%
     paste0("brightness_timeseries", "/", ., make_nb_filename_ending(bts)) %>%
@@ -288,12 +334,13 @@ brightness_timeseries_file <- function(path, def, frames_per_set,
 #' img <- ijtiff::read_tif(system.file('extdata', '50.tif', package = 'nandb'))
 #' ijtiff::write_tif(img, 'img1.tif')
 #' ijtiff::write_tif(img, 'img2.tif')
-#' brightness_folder(def = "B", tau = NA, thresh = "Huang", parallel = 2)
+#' brightness_folder(def = "B", thresh = "Huang")
 #' }
 #' @export
 brightness_folder <- function(folder_path = ".", def,
-                              tau = NULL, thresh = NULL, filt = NULL,
-                              correct = FALSE,
+                              thresh = NULL,
+                              detrend = FALSE, quick = FALSE,
+                              filt = NULL,
                               s = 1, offset = 0, readout_noise = 0,
                               parallel = FALSE) {
   if (startsWith("epsilon", tolower(def))) def <- "epsilon"
@@ -302,11 +349,14 @@ brightness_folder <- function(folder_path = ".", def,
   on.exit(setwd(init_dir))
   setwd(folder_path)
   file_names <- list.files(pattern = "\\.tiff*$")
-  purrr::map(file_names, brightness_file, def = def, tau = tau,
-             thresh = thresh, correct = correct,
-             filt = filt, s = s, offset = offset,
-             readout_noise = readout_noise,
-             parallel = parallel) %>%
+  purrr::map(file_names, brightness_file,
+    def = def,
+    thresh = thresh,
+    detrend = detrend, quick = quick,
+    filt = filt, s = s, offset = offset,
+    readout_noise = readout_noise,
+    parallel = parallel
+  ) %>%
     magrittr::set_names(filesstrings::before_last_dot(file_names)) %>%
     invisible()
 }
@@ -331,14 +381,14 @@ brightness_folder <- function(folder_path = ".", def,
 #' img <- ijtiff::read_tif(system.file('extdata', '50.tif', package = 'nandb'))
 #' ijtiff::write_tif(img, 'img1.tif')
 #' ijtiff::write_tif(img, 'img2.tif')
-#' brightness_timeseries_folder(def = "e", tau = NA, thresh = "Huang",
-#'                               frames_per_set = 20, parallel = 2)
+#' brightness_timeseries_folder(def = "e", thresh = "tri", frames_per_set = 20)
 #' }
 #' @export
 brightness_timeseries_folder <- function(folder_path = ".", def,
-                                         frames_per_set, tau = NULL,
-                                         thresh = NULL, filt = NULL,
-                                         correct = FALSE,
+                                         frames_per_set, overlap = FALSE,
+                                         thresh = NULL,
+                                         detrend = FALSE, quick = FALSE,
+                                         filt = NULL,
                                          s = 1, offset = 0, readout_noise = 0,
                                          parallel = FALSE) {
   if (startsWith("epsilon", tolower(def))) def <- "epsilon"
@@ -347,13 +397,15 @@ brightness_timeseries_folder <- function(folder_path = ".", def,
   on.exit(setwd(init_dir))
   setwd(folder_path)
   file_names <- list.files(pattern = "\\.tif")
-  purrr::map(file_names, brightness_timeseries_file, def = def,
-             frames_per_set = frames_per_set,
-             tau = tau, thresh = thresh, filt = filt,
-             correct = correct,
-             s = s, offset = offset, readout_noise = readout_noise,
-             parallel = parallel) %>%
+  purrr::map(file_names, brightness_timeseries_file,
+    def = def,
+    frames_per_set = frames_per_set, overlap = overlap,
+    thresh = thresh,
+    detrend = detrend, quick = quick,
+    filt = filt,
+    s = s, offset = offset, readout_noise = readout_noise,
+    parallel = parallel
+  ) %>%
     magrittr::set_names(filesstrings::before_last_dot(file_names)) %>%
     invisible()
 }
-
